namespace backgroundworder_test {

	using System;
	using System.Collections.Generic;
	using System.ComponentModel;
	using System.Data;
	using System.Drawing;
	using System.Text;
	using System.Windows.Forms;


	/////////////////////////////////////////////////////////////////////////


	public partial class Form1 : Form {

		public Form1() {
			InitializeComponent();
		}


		/////////////////////////////////////////////////////////////////////


		private void Form1_Load( object sender, EventArgs e ) {

			//
			// Set Event Handlers
			//

			backgroundWorker1.DoWork += new DoWorkEventHandler( backgroundWorker1_DoWork );
			backgroundWorker1.ProgressChanged += new ProgressChangedEventHandler( backgroundWorker1_ProgressChanged );
			backgroundWorker1.RunWorkerCompleted += new RunWorkerCompletedEventHandler( backgroundWorker1_RunWorkerCompleted );
		
			//
			// Initialize Controls
			//

			comboBox1.SelectedIndex = 1;

		}


		/////////////////////////////////////////////////////////////////////


		void backgroundWorker1_RunWorkerCompleted( object sender, RunWorkerCompletedEventArgs e ) {

			//
			// Set Buttons
			//

			btnDoWork.Enabled = true;
			btnCancel.Enabled = false;
			
			//
			// On Error or Cancel
			//

			if (e.Error != null) {
				MessageBox.Show( e.Error.Message );
				return;
			} else if (e.Cancelled) {
				MessageBox.Show( "Operation has been cancelled." );
				return;
			}
			
			//
			// Complete
			//

			WORK_RESULT res = e.Result as WORK_RESULT;

			if (res == null) {
				throw new Exception( "Invalid Context" );
			}

			MessageBox.Show(
				string.Format( "Work Completed. {0} - {1}", res.value, res.msg )
			);

		}


		/////////////////////////////////////////////////////////////////////


		void backgroundWorker1_ProgressChanged( object sender, ProgressChangedEventArgs e ) {

			progressBar1.Value = e.ProgressPercentage;
		
		}


		/////////////////////////////////////////////////////////////////////


		private void backgroundWorker1_DoWork( object sender, DoWorkEventArgs args ) {

			// Get Parameter
			int nMillSec = (int) args.Argument;

			// Return value
			WORK_RESULT res = new WORK_RESULT();

			// Get Worker Object
			BackgroundWorker worker = sender as BackgroundWorker;
			if (worker == null) {
				throw new Exception( "Invalid Context" );
			}

			// Do Time-Consuming Work
			for (int i = 0; i < 20; i++) {
				// Cancel
				if (worker.CancellationPending) {
					args.Cancel = true;
					return;
				}

				// Time-consuming work
				System.Threading.Thread.Sleep( 1 * nMillSec );
				worker.ReportProgress( (i + 1) * 5 );
			}

			// Result
			res.value = 20;
			res.msg = "Completed";
			args.Result = res;
		}


		/////////////////////////////////////////////////////////////////////


		private void button1_Click( object sender, EventArgs e ) {

			btnDoWork.Enabled = false;
			btnCancel.Enabled = true;

			int nParam = int.Parse( comboBox1.Text );

			progressBar1.Value = 0;
			backgroundWorker1.RunWorkerAsync( nParam );

		}


		/////////////////////////////////////////////////////////////////////


		private void button2_Click( object sender, EventArgs e ) {

			if (DialogResult.Yes == MessageBox.Show(
				"Are you sure you want to cancel this operation?",
				"Confirmation",
				MessageBoxButtons.YesNo )) {
			
				backgroundWorker1.CancelAsync();
			
			}

		}
	}


	//
	// WORK RESULT
	//

	public class WORK_RESULT {
		public int value;
		public string msg;
	}

}